/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2013 Raytrix GmbH. All rights reserved.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#ifdef LINUX
	#define DLL_API
	#define RX_STATIC
#else
    #ifdef RX_RAYCAMAPI_EXPORTS
		#define DLL_API __declspec(dllexport)
    #else
		#define DLL_API __declspec(dllimport)
    #endif
#endif

#pragma managed(push, off)

#include "Rx.RayCam.Interface/IDriver.h"
#include "Rx.RayCam.Interface/IStatusHandlerFunction.h"
#include "Rx.Interop.Runtime31/IException.h"
#include "Rx.Interop.Runtime28/RxImageTypes.h"

#include <string>

namespace Rx
{
	namespace RayCam
	{
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	Creates the driver object. The default order to create a camera would be:
		///
		/// 	1. Call Rx::RayCam::IDriver::InitDriver() on the Rx::RayCam::IDriver object. 2. Call
		/// 	Rx::RayCam::IDriver::DiscoverDevices() to get the number of connected devices. 3. Call
		/// 	Rx::RayCam::IDriver::CreateDevice() with the index of the device that should be opened.
		/// </summary>
		///
		/// <returns> The new driver. </returns>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		DLL_API Rx::RayCam::IDriver* CreateDriver(void);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	Releases and cleans up the driver and disconnects all attached cameras.
		/// </summary>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		DLL_API void ReleaseDriver(void);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// \brief
		/// 	Registers a status message callback Function. This function will be called when a new Status Message is fired.
		///
		/// \param	pFunc		 	The callback function.
		/// \param [in]	pvContext	A Context Pointer which is forwarded to the callback function call.
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		DLL_API void AddStatusMessageHandler(TStatusMessageCallback pFunc, void* pvContext);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// \brief
		/// 	Removes a registered message callback function.
		///
		/// \param	pFunc		 	The function.
		/// \param [in]	pvContext	If non-null, context for the pv.
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		DLL_API void RemoveStatusMessageHandler(TStatusMessageCallback pFunc, void* pvContext);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	Set the Info log Level.
		///
		/// 	Default level is 0 Extended info level is 1 Full info output is 2.
		///
		/// 	\attention There is no need to call this function. It's only for debugging.
		/// </summary>
		///
		/// <param name="iErrorLevel"> Zero-based index of the error level. </param>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		DLL_API void SetInfoLogLevel(int iErrorLevel);

		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	Adds an information log message.
		///
		/// 	\attention There is no need to call this function. It's only for debugging.
		/// </summary>
		///
		/// <param name="iInfoLevel"> Zero-based index of the information level. </param>
		/// <param name="sMsg">		  The message. </param>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		DLL_API void AddInfoLogMessage(int iInfoLevel,  std::string sMsg);
	}
}

#pragma managed(pop)
