/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2015 Raytrix GmbH. All rights reserved.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#pragma managed(push, off)

#include "ECamProperty.h"

#ifdef LINUX
	#define __cdecl __attribute__((__cdecl__))
#endif

namespace Rx
{
	namespace RayCam
	{
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 	This class represents a callback image.
		/// </summary>
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		class IImageProperty
		{
		public:

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Can be used to copy the IImageProperty by the extended Callback function of IDevice. It is recommended to copy the 
			/// 	image data as well as the IImageProperty into a separate buffer to handle the IDevice callback as fast as possible.
			/// </summary>
			///
			/// <returns> Null if it fails, else. </returns>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual IImageProperty* Copy(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Used to destroy the object created with the copy function.
			/// </summary>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void Destroy(void) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a image property of type int or bool (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, int& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a image property of type double.
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, double& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets a image property of type long long (0 represents false).
			/// </summary>
			///
			/// <param name="eProp">    The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue"> [in,out] The value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, long long& theValue) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a image property of type int.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue1"> [out] The first value. </param>
			/// <param name="theValue2"> [out] The second value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, int& theValue1, int& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a image property of type double.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue1"> [out] The first value. </param>
			/// <param name="theValue2"> [out] The second value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, double& theValue1, double& theValue2) = 0;

			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			/// <summary>
			/// 	Gets the min and max value of a image property of type long long.
			/// </summary>
			///
			/// <param name="eProp">	 The Rx::RayCam::ECamProperty::ID property. </param>
			/// <param name="theValue1"> [out] The first value. </param>
			/// <param name="theValue2"> [out] The second value. </param>
			/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			virtual void GetProperty(ECamProperty::ID eProp, long long& theValue1, long long& theValue2) = 0;
		};
	}
}

#pragma managed(pop)
