#include <iostream>
#include <cstring>
#include <mutex>

#define RX_STATIC
#include "Rx.RayCam.API/RayCamAPI.h"


//#include "Rx.Core/RxExceptions.h"

#include<stdio.h>
#include<stdlib.h>
#include<X11/X.h>
#include<X11/Xlib.h>

#include <GL/glew.h>

#include<GL/gl.h>
#include<GL/glx.h>
#include<GL/glu.h>



// GL Context vars
Display                 *g_xDisplay;
Window                  g_rootWindow;
GLint                   g_xContextAttributes[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
XVisualInfo             *g_xVisualInfo;
Colormap                g_colorMap;
XSetWindowAttributes    g_swa;
Window                  g_window;
GLXContext              g_glcontext;



using namespace std;

int g_iWidth;
int g_iHeight;
int g_ePixType;
int g_eDatType;
unsigned int g_uDataSize = 0;
unsigned char* g_pData = 0;

float fPixelTransferSpeed = 0.5;
bool bSingleTrigger = false;

Rx::RayCam::IDevice* xMyDevice = 0;

std::mutex xMutex;

void renderScene(void)
{
	static int iTexID = 0;
	glEnable (GL_TEXTURE_2D);

	if (iTexID == 0)
	{
		glGenTextures(1, &iTexID);
	}

	xMutex.lock();
	if (g_pData != 0)
	{
		if (g_iWidth == 1 && g_iHeight == 1)
		{
			int iTextureFromSDK = (int)g_pData[0];

			glBindTexture(GL_TEXTURE_2D, iTextureFromSDK);
		}
		else
		{

			glBindTexture(GL_TEXTURE_2D, iTexID);

			if (g_ePixType == Rx::Interop::Runtime28::EPixelType::RGB)
			{
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, g_iWidth, g_iHeight, 0,	GL_RGB, GL_UNSIGNED_BYTE, g_pData);
			}
			else if (g_ePixType == Rx::Interop::Runtime28::EPixelType::RGBA)
			{
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, g_iWidth, g_iHeight, 0,	GL_RGBA, GL_UNSIGNED_BYTE, g_pData);
			}
			else
			{
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, g_iWidth, g_iHeight, 0,	GL_LUMINANCE, GL_UNSIGNED_BYTE, g_pData);
			}
			//glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, g_iWidth, g_iHeight, 0,	GL_RGB, GL_UNSIGNED_BYTE, g_pData);

			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		}
		//glGenerateMipmap(GL_TEXTURE_2D);
	}
	xMutex.unlock();



	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glBegin (GL_QUADS);
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f(1, -1, 0);

	glTexCoord2f(0.0f, 1.0f);
	glVertex3f(-1, -1, 0.0);

	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(-1, 1, 0.0);

	glTexCoord2f(1.0f, 0.0f);
	glVertex3f(1, 1, 0.0);
	glEnd();

}





void myCallback(void* callbackOwner,
		unsigned int uWidth, unsigned int uHeight, unsigned int iPixelType,
		unsigned int iDataType, void* pData)
{

	cout << "Received Image with size: " << uWidth << "x" << uHeight << " PixFmt: " << iPixelType << " DataType: " << iDataType <<  endl;

	xMutex.lock();

	g_iWidth = uWidth;
	g_iHeight = uHeight;
	g_eDatType = iDataType;
	g_ePixType = iPixelType;

	int iChannels = 1;
	int iSizeChannel = 1;

	if (g_eDatType == Rx::Interop::Runtime28::EDataType::UByte || g_eDatType == Rx::Interop::Runtime28::EDataType::Byte )
	{
		iSizeChannel = 1;
	}
	else if (g_eDatType == Rx::Interop::Runtime28::EDataType::UShort || g_eDatType == Rx::Interop::Runtime28::EDataType::Short)
	{
		iSizeChannel = 2;
	}
	else if (g_eDatType == Rx::Interop::Runtime28::EDataType::UInt || g_eDatType == Rx::Interop::Runtime28::EDataType::Int )
	{
		iSizeChannel = 4;
	}


	if (g_ePixType == Rx::Interop::Runtime28::EPixelType::RGB)
	{
		iChannels = 3;
	}
	else if (g_ePixType == Rx::Interop::Runtime28::EPixelType::RGBA)
	{
		iChannels = 4;
	}
	else
	{
		iChannels = 1;
	}

	if (g_uDataSize == 0 || g_uDataSize != (uHeight * uWidth * iChannels * iSizeChannel))
	{
		delete g_pData;
		g_uDataSize = uHeight * uWidth * iChannels* iSizeChannel;
		g_pData = new unsigned char[g_uDataSize];

	}


	memcpy(g_pData, pData, g_uDataSize);

	xMutex.unlock();
}


void keyFunc(unsigned char key)
{
	double dExp;
	int dFocus;
	int iTriggerMode;

	double fFps;


	double dFramerateMax, dFramerateMin, dExposureMax, dExposureMin;

	switch (key)
	{

	case 'x':
		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Framerate, fFps);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, max(fFps - 0.1,dFramerateMin ));
		break;

	case 'c':
		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Framerate, fFps);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, fFps + 0.1);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, min(fFps + 0.1,dFramerateMax ));
		break;


	case 'q':
		xMyDevice->StopStream();
		xMyDevice->Close();
		exit(0);
		break;

	case '+':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Exposure, dExp);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExp * 1.1);
		break;

	case '0':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Focus, dFocus);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Focus, dFocus + 15);
		break;

	case '9':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Focus, dFocus);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Focus, dFocus - 15);
		break;

	case '-':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Exposure, dExp);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExp * 0.9);
		break;

	case 't':
		if (bSingleTrigger == false)
		{
			xMyDevice->StopStream();
			xMyDevice->SetProperty(Rx::RayCam::ECamProperty::GlobalStart, 1);
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
			bSingleTrigger = true;
		}

		xMyDevice->Trigger();
		break;

	case 's':
		if (bSingleTrigger == true)
		{
			xMyDevice->StopStream();
			xMyDevice->SetProperty(Rx::RayCam::ECamProperty::GlobalStart, 0);
			xMyDevice->StartStream(
					Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
			bSingleTrigger = false;
		}
		break;

	case '1':
		xMyDevice->StopStream();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::FULLRES_ZOOM1_BIT8);	// Set preferred Video mode

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExposureMax);	// Set exposure in seconds

		if (bSingleTrigger)
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
		}
		else
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
		}
		break;

	case '2':
		xMyDevice->StopStream();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::UHD_ZOOM1_BIT8);	// Set preferred Video mode

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExposureMax);	// Set exposure in seconds

		if (bSingleTrigger)
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
		}
		else
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
		}
		break;

	case '3':
		xMyDevice->StopStream();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::HD_ZOOM1_BIT8);	// Set preferred Video mode

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExposureMax);	// Set exposure in seconds


		if (bSingleTrigger)
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
		}
		else
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
		}

		break;

	}
}


bool CreateWindow(GLXContext xContext)
{
	g_xDisplay = XOpenDisplay(NULL);
 
 	if(g_xDisplay == NULL) {
 		printf("\n\tcannot connect to X server\n\n");
        	exit(0);
 	}
		
	 g_rootWindow = DefaultRootWindow(g_xDisplay);

	 g_xVisualInfo = glXChooseVisual(g_xDisplay, 0, g_xContextAttributes);

	 if(g_xVisualInfo == NULL) {
		printf("\n\tno appropriate visual found\n\n");
		exit(0);
	 } 
	 else {
		printf("\n\tvisual %p selected\n", (void *)g_xVisualInfo->visualid); /* %p creates hexadecimal output like in glxinfo */
	 }


	 g_colorMap = XCreateColormap(g_xDisplay, g_rootWindow, g_xVisualInfo->visual, AllocNone);

	 g_swa.colormap = g_colorMap;
	 g_swa.event_mask = ExposureMask | KeyPressMask;
	 
	 g_window = XCreateWindow(g_xDisplay, g_rootWindow, 0, 0, 600, 600, 0, g_xVisualInfo->depth, InputOutput, g_xVisualInfo->visual, CWColormap | CWEventMask, &g_swa);

	 XMapWindow(g_xDisplay, g_window);
	 XStoreName(g_xDisplay, g_window, "Raycam Sharead Context example");
	 
	 g_glcontext = glXCreateContext(g_xDisplay, g_xVisualInfo, xContext, GL_TRUE);
	 glXMakeCurrent(g_xDisplay, g_window, g_glcontext);
}


void MainLoop()
{
	XEvent                  xev;
	KeySym key;
	char text[255];
	XWindowAttributes       gwa;

	int x11_fd = ConnectionNumber(g_xDisplay);
	fd_set in_fds;

	struct timeval tv;

	while(1) 
	{
		 // Create a File Description Set containing x11_fd
		FD_ZERO(&in_fds);
		FD_SET(x11_fd, &in_fds);

		// Set our timer.  One second sounds good.
		tv.tv_usec = 33;
		tv.tv_sec = 0;

		// Wait for X Event or a Timer
		if (select(x11_fd+1, &in_fds, 0, 0, &tv))
		{
				//event
		}
		else
		{

			//timer
			glXMakeCurrent(g_xDisplay, g_window, g_glcontext);
			XGetWindowAttributes(g_xDisplay, g_window, &gwa);
			glViewport(0, 0, gwa.width, gwa.height);
			renderScene();
		    glXSwapBuffers(g_xDisplay, g_window);
			// Handle timer here
			//printf("Timer Fired!\n");
		}


		while(XPending(g_xDisplay))
		{

			XNextEvent(g_xDisplay, &xev);
		
		if(xev.type == KeyPress)
		{
			// translate to ordinary ascii
			XLookupString(&xev.xkey,text,255,&key,0);
			keyFunc(text[0] );
		}
		}
	} /* this closes while(1) { */
}



void HandleException(Rx::Interop::Runtime31::IException* ex)
{
	cout << ex->GetFile() << " " << ex->GetFunction() << " " << ex->GetLine() << " : " << ex->GetMessageText() << endl;

	if (ex->GetInnerException() != 0)
	{
		HandleException(ex->GetInnerException());
	}
}


int main(int argc, char **argv)
{

	for (int i = 1; i < argc; i++)
	{
		if (argv[i][0] == 't')
		{
			fPixelTransferSpeed = atof(argv[i+1]);
		}
	}

	try
	{
		cout << "Create Driver" << endl;

		Rx::RayCam::IDriver* xMyDriver = Rx::RayCam::CreateDriver();
		Rx::RayCam::SetInfoLogLevel(99);

		if (xMyDriver == 0)
		{
			cout << "error, driver is null" << endl;
			return 0;
		}

		cout << "Initialize driver" << endl;
		xMyDriver->InitDriver();

		cout << "Discover devices" << endl;
		int iDeviceCnt = 0;
		xMyDriver->DiscoverDevices(iDeviceCnt);

		cout << "found " << iDeviceCnt << " Devices" << endl;

		cout << "Create Device" << endl;
		xMyDevice = xMyDriver->CreateDevice(0);

		if (xMyDevice == 0)
		{
			cout << "Error on CreateDevice" << endl;
			return 0;
		}

		cout << "Open Device" << endl;

		xMyDevice->Open();

		long long llContext;
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::OpenGLContextHandle_Windows, llContext);

		GLXContext* pxContext = (GLXContext*)llContext;

		CreateWindow(*pxContext);


		xMyDevice->RegisterCallback(xMyDevice, myCallback);

	//	cout << "press any key to start stream" << endl;
	//	getchar();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::ImageFormat, Rx::RayCam::EImageFormat::RGBA);

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::ImageTransferMode, Rx::RayCam::EImageTransferMode::GLTextureID);

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::TransferSpeed,fPixelTransferSpeed);


		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::HD_ZOOM1_BIT8);	// Set preferred Video mode

		double dFramerateMax, dFramerateMin, dExposureMax, dExposureMin;
		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate,dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, min(0.133,dExposureMax));	// Set exposure in seconds

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::AutoGain, 1);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::AutoExposure, 1);



		xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);


		MainLoop();

		xMyDevice->StopStream();

		xMyDevice->Close();

		xMyDriver->DestroyDevice(xMyDevice);

	}
	catch(Rx::Interop::Runtime31::IException& ex)
	{
		cout << "catched Exception:  " << endl;
		HandleException(&ex);
	}

}
